#include "dynamics.h"
#include <iostream>
#include <math.h>

using namespace std;

/////////////////////////
// accessors
/////////////////////////

int Dynamics::addBall(const Vec& ballPos, const Vec& velocity, float mass, float radius){
	positions.push_back(ballPos);
	velocities.push_back(velocity);
	masses.push_back(mass);
	if (mass>0) {
		invMasses.push_back( 1/mass);
	} else {
		invMasses.push_back(0.0);
	}
	radiuses.push_back(radius);

	return positions.size()-1;
}

int Dynamics::addSpring(int ball1, int ball2, double stiffness, double initLength, double dampingFactor){
	springs.push_back(Spring(ball1, ball2, stiffness, initLength, dampingFactor));
	return springs.size();
}

void Dynamics::setGravity(const Vec& grav){
	gravity = grav;
}

void Dynamics::setBallPos( int boule, const Vec& pos){
	if (boule<0 || boule>=positions.size()){
		cerr << "Bad ball index !!" << endl;
	}
	positions[boule] = pos;
}



/////////////////////////
// methods
/////////////////////////

void Dynamics::animate(double dt){

		////////////////////
		// Force computations
		
		int nbBalls = positions.size();
		vector<Vec> forces(nbBalls);
		for( int i=0; i<nbBalls; ++i ) {
				// poids
				forces[i] = forces[i] + masses[i] * gravity;
		}
		
		for( int i=0; i<springs.size(); ++i ) {
				// ressorts amortis
				springs[i].addForce(forces, positions, velocities);
		}
		
		///////////////////////////
		// Acceleration computations :
		
		for( int i=0; i<nbBalls; ++i ) {
				// les acclrations sont stockes dans forces
				forces[i] = invMasses[i] * forces[i];
		}
	
		///////////////////////
		// Integration scheme :

		for( int i=0; i<nbBalls; ++i ) {
				positions[i] = positions[i] + dt * velocities[i];
		}

		////////////////////////////
		// Collisions :
		//
 	//	for( int i=0; i<nbBalls; ++i ){
 	//			// avec le sol
 	//			collisionBallPlane(positions[i], velocities[i], radiuses[i], invMasses[i], groundPosition, groundVelocity, groundNormal, 0.0f, 0.5f);
		//}
		
}



void Dynamics::collisionBallPlane(Vec& x1, Vec& v1, double r1, double invm1,
																	Vec& x2, Vec& v2, Vec& normal, double invm2, double rebound ){
		// don't process fixed objects :
		if( invm1==0 && invm2==0 ) return;
	
		// Plane-ball distance :
		Vec gap = x1 - x2;
		double penetration = (gap * normal) - r1;
		if( penetration > 0 ) return;
		
		// Penetration velocity :
		double vpen = (v1-v2)*normal;
		
		// Correction distribution (the heaviest object moves less) :
		double corr1 = invm1 / (invm1+invm2);
		double corr2 = invm2 / (invm1+invm2);
		
		// rebound :
		corr1 *= (1+rebound);
		corr2 *= (1+rebound);
		
		// position and velocity updates :
		x1 = x1 - (corr1 * penetration) * normal;
		v1 = v1 - (corr1 * vpen) * normal;
		x2 = x2 + (corr2 * penetration) * normal;
		v2 = v2 + (corr2 * vpen) * normal;		
}


void Dynamics::collisionBallBall(Vec& x1, Vec& v1, double r1, double invm1,
																 Vec& x2, Vec& v2, double r2, double invm2,
																 double rebound ){

}
