#ifndef _DYNAMICS_H_
#define _DYNAMICS_H_

#include "vec.h"
#include "spring.h"


class Dynamics {
private:
	// balls
	std::vector<Vec> positions; 
	std::vector<Vec> velocities; 
	std::vector<double> masses; 
	std::vector<double> invMasses; 
	std::vector<double> radiuses;
	
	// ground
	Vec groundPosition;
	Vec groundVelocity;
	Vec groundNormal;

	// forces
	std::vector<Spring> springs;
	Vec gravity;
	float viscosity;


	
public:

	// initialize dynamic animation
	Dynamics()
		: gravity(0.0, -10.0, 0.0), viscosity(1.0),
		groundPosition(0.0, 0.0, 0.0),
		groundVelocity(0.0, 0.0, 0.0),
		groundNormal(0.0, 1.0, 0.0) {}
	
	~Dynamics(){}

	// ACCESSORS
	
	// add a ball to dynamic animation and send back an identifier for this ball
	int addBall(const Vec& ballPos, const Vec& velocity, float mass, float radius);
	// add a spring between two balls
	int addSpring(int ball1, int ball2, double stiffness, double initLength, double dampingFactor);

	// set gravity vector
	void setGravity(const Vec& grav);
	// force the position of a ball
	void setBallPos( int ball, const Vec& pos);
	
	// get ball position
	const Vec& getBallPos(int ball) const {return positions[ball];}
	// get ball radius
	double getBallRadius(int ball) const {return radiuses[ball];}
	// get number of springs
	int getNbSprings() const {return springs.size();}
	// get a spring
	const Spring& getSpring (int spring) const {return springs[spring];}

	// METHODS

	// compute collision between a ball and a plane
	void collisionBallPlane(Vec& x1, Vec& v1, double r1, double invm1,
													Vec& x2, Vec& v2, Vec& normal, double invm2,
													double rebound );

	// compute collision between a ball and a ball
	void collisionBallBall(Vec& x1, Vec& v1, double r1, double invm1,
												 Vec& x2, Vec& v2, double r2, double invm2,
												 double rebound );
	
	// update positions and velocities of dynamic objects after a time step dt
	void animate(double dt);

};

#endif // _DYNAMICS_H_
